/*
 *   Copyright 1992, 1993, 1994 John Melton (G0ORX/N6LYT)
 *              All Rights Reserved
 *
 *   This program is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation; either version 1, or (at your option)
 *   any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program; if not, write to the Free Software
 *   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 */

/*
	message.c

	Prepare a PacSat message for uploading

	John Melton
	G0ORX, N6LYT

	4 Charlwoods Close
	Copthorne
	West Sussex
	RH10 3QZ
	England

	INTERNET:	g0orx@amsat.org
			n6lyt@amsat.org
			john@images.demon.co.uk
			J.D.Melton@slh0613.icl.wins.co.uk

	History:

	0.1	initial version.
	0.2	fixed file length in header.
	0.3	converted to Xaw.
		removed size restriction on message and banner.
*/

#define INSERTCRLF

#define VERSION_STRING "message (version 0.3 by g0orx/n6lyt/g4klx)"

#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <sys/types.h>
#include <fcntl.h>
#include <dirent.h>
#include <time.h>

#include <X11/Intrinsic.h>
#include <X11/StringDefs.h>
#include <X11/Shell.h>
#include <X11/Xaw/Cardinals.h>
#include <X11/Xaw/Form.h>
#include <X11/Xaw/Text.h>
#include <X11/Xaw/AsciiText.h>
#include <X11/Xaw/Label.h>
#include <X11/Xaw/Command.h>
#include <X11/Xaw/MenuButton.h>

#include "xawutils.h"
#include "header.h"
#include "ftl0.h"

Display *dpy;

XtAppContext app_context;

typedef struct
{
	XFontStruct *bold_font, *button_font, *menu_font, *label_font, *text_font;
}
Resources;

Resources  resources;

Widget toplevel, compwindow, quitbutton, preparebutton;
Widget fromlabel, fromtext, tolabel, totext, titlelabel, titletext;
Widget keywordslabel, keywordstext, filetypebutton, filetypelabel;
Widget comptypebutton, comptypelabel, filedesclabel, filedesctext;
Widget mesgsrcbutton, mesgsrclabel, bannerbutton, datawindow;

static XtResource resource_list[] =
{
	{"boldFont", XtCFont, XtRFontStruct, sizeof(XFontStruct *),
		XtOffsetOf(Resources, bold_font), XtRString, XtDefaultFont},
	{"buttonFont", XtCFont, XtRFontStruct, sizeof(XFontStruct *),
		XtOffsetOf(Resources, button_font), XtRString, XtDefaultFont},
	{"menuFont", XtCFont, XtRFontStruct, sizeof(XFontStruct *),
		XtOffsetOf(Resources, menu_font), XtRString, XtDefaultFont},
	{"labelFont", XtCFont, XtRFontStruct, sizeof(XFontStruct *),
		XtOffsetOf(Resources, label_font), XtRString, XtDefaultFont},
	{"textFont", XtCFont, XtRFontStruct, sizeof(XFontStruct *),
		XtOffsetOf(Resources, text_font), XtRString, XtDefaultFont}
};

static Arg shell_args[] =
{
	{XtNtitle,		(XtArgVal)NULL}
};

static Arg form_args[] =
{
	{XtNdefaultDistance,	(XtArgVal)0}
};

static Arg label_args[] =
{
	{XtNfromVert,		(XtArgVal)NULL},
	{XtNlabel,		(XtArgVal)NULL},
	{XtNwidth,		(XtArgVal)0},
	{XtNfont,		(XtArgVal)NULL},
	{XtNborderWidth,	(XtArgVal)0},
	{XtNresize,		False},
	{XtNjustify,		XtJustifyLeft},
	{XtNvertDistance,	(XtArgVal)6},
	{XtNhorizDistance,	(XtArgVal)8},
	{XtNtop,		XtChainTop},
	{XtNbottom,		XtChainTop},
	{XtNleft,		XtChainLeft},
	{XtNright,		XtChainLeft}
};

static Arg button_args[] =
{
	{XtNcallback,		(XtArgVal)NULL},
	{XtNlabel,		(XtArgVal)NULL},
	{XtNfont,		(XtArgVal)NULL},
	{XtNfromHoriz,		(XtArgVal)NULL},
	{XtNfromVert,		(XtArgVal)NULL},
	{XtNvertDistance,	(XtArgVal)6},
	{XtNhorizDistance,	(XtArgVal)8},
	{XtNresize,		False},
	{XtNtop,		XtChainTop},
	{XtNbottom,		XtChainTop},
	{XtNleft,		XtChainLeft},
	{XtNright,		XtChainLeft}
};

static Arg menu_button_args[] =
{
	{XtNlabel,		(XtArgVal)NULL},
	{XtNfromHoriz,		(XtArgVal)NULL},
	{XtNfromVert,		(XtArgVal)NULL},
	{XtNwidth,		(XtArgVal)0},
	{XtNfont,		(XtArgVal)NULL},
	{XtNresize,		False},
	{XtNvertDistance,	(XtArgVal)6},
	{XtNhorizDistance,	(XtArgVal)8},
	{XtNtop,		XtChainTop},
	{XtNbottom,		XtChainTop},
	{XtNleft,		XtChainLeft},
	{XtNright,		XtChainLeft},
	{XtNjustify,		XtJustifyCenter}
};

static Arg text_args[] =
{
	{XtNfromHoriz,		(XtArgVal)NULL},
	{XtNfromVert,		(XtArgVal)NULL},
	{XtNlength,		(XtArgVal)0},
	{XtNwidth,		(XtArgVal)0},
	{XtNstring,		(XtArgVal)NULL},
	{XtNfont,		(XtArgVal)NULL},
	{XtNvertDistance,	(XtArgVal)6},
	{XtNhorizDistance,	(XtArgVal)8},
	{XtNtop,		XtChainTop},
	{XtNbottom,		XtChainTop},
	{XtNleft,		XtChainLeft},
	{XtNright,		XtChainLeft},
	{XtNeditType,		XawtextEdit},
	{XtNtype,		XawAsciiString},
	{XtNresize,		False},
	{XtNuseStringInPlace,	True}
};

static Arg window_args[] =
{
	{XtNfromVert,		(XtArgVal)NULL},
	{XtNbackground,		(XtArgVal)NULL},
	{XtNfont,		(XtArgVal)NULL},
	{XtNwidth,		(XtArgVal)640},
	{XtNheight,		(XtArgVal)150},
	{XtNvertDistance,	(XtArgVal)6},
	{XtNhorizDistance,	(XtArgVal)0},
	{XtNtop,		XtChainTop},
	{XtNbottom,		XtChainBottom},
	{XtNleft,		XtChainLeft},
	{XtNright,		XtChainRight},
	{XtNeditType,		XawtextEdit},
	{XtNtype,		XawAsciiString},
	{XtNwrap,		XawtextWrapWord},
	{XtNscrollVertical,	XawtextScrollAlways}
};

MenuEntry comp_type_menu[] =
{
	{"Not Compressed",	"comptype1"},
	{"PKARC",		"comptype2"},
	{"PKZIP",		"comptype3"},
	{"LHARC",		"comptype4"}
};

MenuEntry mesg_src_menu[] =
{
	{"Inline Text",		"mesgsrc1"},
	{"File",		"mesgsrc2"}
};

MenuEntry file_type_menu[] =
{
	{"ASCII text",			"filetype1"},
	{"RLI/MBL message(single)",	"filetype2"},
	{"RLI/MBL message(multiple)",	"filetype3"},
	{"UoSAT WOD",			"filetype4"},
	{"PacSat WOD",			"filetype5"},
	{"UoSAT CPE",			"filetype6"},
	{"MS/PC-DOS .exe",		"filetype7"},
	{"MS/PC-DOS .com",		"filetype8"},
	{"Keps (NASA 2-line)",		"filetype9"},
	{"Keps (AMSAT)",		"filetype10"},
	{"Error logs",			"filetype11"},
	{"Activity logs",		"filetype12"},
	{"Other",			"filetype13"}
};

char myCall[32];
char toCall[32];
char title[80];
char keywords[80];
char filetype_desc[80];
char banner[128];

#define MAXHEADER 1024
unsigned char header[MAXHEADER];
int nHeader;

unsigned char	szHeaderId[2];
unsigned long	nFileNumber;
char		szFileName[9] = "        ";
char		szFileExt[4] = "   ";
unsigned long	nFileSize     = 0;
unsigned long	nCreateTime   = 0;
unsigned long	nModifiedTime = 0;
unsigned char	nSEUflag      = 0;
unsigned char	nFileType;
unsigned int	nBodyChecksum   = 0;
unsigned int	nHeaderChecksum = 0;
unsigned int	nBodyOffset;
unsigned char	szSource[32];
unsigned char	szUploader[7] = "      ";
unsigned long	nUploadTime    = 0;
unsigned char	nDownloadCount = 0;
char		szDestination[32];
char		szDownloader[7] = "      ";
unsigned long	nDownloadTime  = 0;
unsigned long	nExpireTime    = 0;
unsigned char	nPriority;
unsigned char	nCompressionType = 0;
char		cBBSMessageType = ' ';
char		szBullitinId[16] = "";


char		szTitle[255];
char		szKeywords[255];
char		szDescription[255] = "";
char		szCompressionDescription[255] = "";
char		szSourceFile[128];
int		source   = 0;
int		fileType = 0;
char		*szText;

int PrepareFile(int uploadSeq, int source);

void QuitCb(Widget w, XtPointer client_data, XtPointer call_data)
{
	XtDestroyApplicationContext(app_context);

	exit(0);
}

void BannerCb(Widget w, XtPointer client_data, XtPointer call_data)
{
	int fd;
	char text[1000];
	int bytes;
	XawTextPosition pos, oldpos;
	XawTextBlock tt;
	Arg args[1];
	String string;
	
	if ((fd = open(banner, O_RDONLY)) != -1)
	{
		XtSetArg(args[0], XtNstring, &string);
		XtGetValues(datawindow, args, 1);
		pos = strlen(string);

		oldpos = XawTextGetInsertionPoint(datawindow);
	
		while ((bytes = read(fd, text, 1000)) > 0)
		{
			XawTextSetInsertionPoint(datawindow, pos);
		
			tt.firstPos = 0;
			tt.ptr      = text;
			tt.length   = bytes;
			tt.format   = FMT8BIT;

			XawTextReplace(datawindow, pos, pos, &tt);
			pos += bytes;
		}
		
		close(fd);

		XawTextSetInsertionPoint(datawindow, oldpos);
	}
}

void PrepareCb(Widget w, XtPointer client_data, XtPointer call_data)
{
	int t;
	char temp[80];
	char *src;
	Arg args[1];

	t = (int)time((time_t *)&t);

	/* extract data from the panel */
	strcpy(szSource,      myCall);
	strcpy(szDestination, toCall);
	strcpy(szTitle,       title);
	sprintf(szKeywords,  "<MGSS> %s", keywords);

	XtSetArg(args[0], XtNstring, &src);
	XtGetValues(datawindow, args, 1);

#ifdef INSERTCRLF
	{
		int  length;
		char *dest;

		length = strlen(src);

		for (dest = src; *dest != '\0'; dest++)
			if (*dest == '\n')
				length++;

		if ((szText = XtMalloc(length + 1)) == NULL)
		{
			perror("PrepareCb");
			return;
		}

		dest = szText;

		while (*src)
		{
			if (*src == '\n') *dest++ = '\r';
			*dest++ = *src++;
		}

		*dest++ = '\0';
	}
#else
	szText = src;
#endif
	if (fileType == 12)
	{
		fileType = 255;
		strcpy(szCompressionDescription, filetype_desc);
	}

	/* prepare the header and the data */
	if (!PrepareFile(t, source))
	{
		sprintf(temp, "Message prepared: file %x.upl", t);
		MessageBox(temp);
	}

#ifdef INSERTCRLF
	XtFree(szText);
#endif
}

void HeaderOut(int v)
{
	if (nHeader < MAXHEADER)
	{
		header[nHeader] = v;
		nHeader++;
	}
}

void Header(unsigned int id, char *string, int length)
{
	int i;

	HeaderOut(id);
	HeaderOut(0);
	HeaderOut(length);

	for (i = 0; i < length; i++)
		HeaderOut(string[i]);
}

int PrepareFile(int uploadSeq, int source)
{
	int hSource, hOut = 0;
	char szOut[128];
	int i, updateFileSize, updateBodyChecksum, updateHeaderChecksum;
	int updateBodyOffset, savedHeaderOffset, nBytes;
	unsigned char buffer[512];

	nHeaderChecksum = 0;
	nBodyChecksum   = 0;
	nFileSize       = 0;
	nBodyOffset     = 0;

	switch (source)
	{
		case 0:
			sprintf(szSourceFile, "%x.txt", uploadSeq);
			break;
		case 1:
			i = strlen((char *)szText) - 1;

			while (i > 0 && (szText[i] == '\n' || szText[i] == '\r'))
			{
				szText[i] = '\0';
				i--;
			}

			strcpy(szSourceFile, szText);
			i = strlen((char *)szSourceFile) - 1;
			while (i > 0 && szSourceFile[i] != '/')
				i--;
			if (szSourceFile[i] == '/')
				i++;
			if (i != 0)
				strcpy(szSourceFile, szSourceFile + i);
			break;
	}

	sprintf(szOut, "%x.upl",  uploadSeq);

	nHeader = 0;

	HeaderOut(0xAA);
	HeaderOut(0x55);
	Header(0x01, (char *)&nFileNumber, 4);
	Header(0x02, (char *)szFileName, 8);
	Header(0x03, (char *)szFileExt, 3);
	updateFileSize = nHeader;
	Header(0x04, (char *)&nFileSize, 4);
	Header(0x05, (char *)&nCreateTime, 4);
	Header(0x06, (char *)&nModifiedTime, 4);
	Header(0x07, (char *)&nSEUflag, 1);
	Header(0x08, (char *)&nFileType, 1);
	updateBodyChecksum = nHeader;
	Header(0x09, (char *)&nBodyChecksum, 2);
	updateHeaderChecksum = nHeader;
	Header(0x0A, (char *)&nHeaderChecksum, 2);
	updateBodyOffset = nHeader;
	Header(0x0B, (char *)&nBodyOffset, 2);

	Header(0x10, (char *)szSource, strlen((char *)szSource));
	Header(0x11, (char *)szUploader, 6);
	Header(0x12, (char *)&nUploadTime, 4);
	Header(0x13, (char *)&nDownloadCount, 1);
	Header(0x14, (char *)szDestination, strlen((char *)szDestination));
	Header(0x15, (char *)szDownloader, 6);
	Header(0x16, (char *)&nDownloadTime, 4);
	Header(0x17, (char *)&nExpireTime, 4);
	Header(0x18, (char *)&nPriority, 1);
	Header(0x19, (char *)&nCompressionType, 1);
	Header(0x22, (char *)szTitle, strlen((char *)szTitle));
	/* Header(0x20, (char *)&cBBSMessageType, 1); */
	/* Header(0x21, (char *)szBullitinId, 0); */
	if (strlen((char *)szKeywords) != 0)
		Header(0x23, (char *)szKeywords, strlen((char *)szKeywords));
	/* Header(0x24, (char *)szDescription, strlen((char *)szDescription)); */
	if (fileType == 255)
		Header(0x25, (char *)szCompressionDescription, strlen((char *)szCompressionDescription));

	Header(0x26, (char *)szSourceFile, strlen(szSourceFile));

	HeaderOut(0x00);
	HeaderOut(0x00);
	HeaderOut(0x00);

	savedHeaderOffset = nHeader;
	nBodyOffset = nHeader;
	nHeader = updateBodyOffset;
	Header(0x0B, (char *)&nBodyOffset, 2);
	nHeader = savedHeaderOffset;

	nFileSize = nHeader;

	switch (source)
	{
		case 0:
			if ((hOut = open(szOut, O_RDWR | O_CREAT | O_TRUNC, 0666)) == -1)
			{
				perror(szOut);
				return -1;
			}
			lseek(hOut, nBodyOffset, SEEK_SET);
			write(hOut, szText, strlen(szText));
			for (i = 0; i < strlen(szText); i++)
				nBodyChecksum = nBodyChecksum + (unsigned int)szText[i];
			nFileSize = nHeader + strlen(szText);
			break;
		case 1:
			if ((hSource = open(szText, O_RDONLY)) == -1)
			{
				perror(szText);
				return -1;
			}
			if ((hOut = open(szOut, O_RDWR | O_CREAT | O_TRUNC, 0666)) == -1)
			{
				close(hSource);
				perror(szOut);
				return -1;
			}
			lseek(hOut, nBodyOffset, SEEK_SET);
			while ((nBytes = read(hSource, (char *)buffer, sizeof(buffer))) > 0)
			{
				nFileSize = nFileSize + nBytes;
				write(hOut, (char *)buffer, nBytes);
				for (i = 0; i < nBytes; i++)
					nBodyChecksum = nBodyChecksum + (unsigned int)buffer[i];
			}
			close(hSource);
			break;
	}

	savedHeaderOffset = nHeader;
	nHeader = updateBodyChecksum;
	Header(0x09, (char *)&nBodyChecksum, 2);
	nHeader = savedHeaderOffset;

	savedHeaderOffset = nHeader;
	nHeader = updateFileSize;
	Header(0x04, (char *)&nFileSize, 4);
	nHeader = savedHeaderOffset;

	for (i = 0; i < nHeader; i++)
		nHeaderChecksum = nHeaderChecksum + (unsigned int)header[i];
		
	savedHeaderOffset = nHeader;
	nHeader = updateHeaderChecksum;
	Header(0x0A, (char *)&nHeaderChecksum, 2);
	nHeader = savedHeaderOffset;

	lseek(hOut, 0L, SEEK_SET);
	write(hOut, header, nHeader);
	close(hOut);

	return 0;
}

void CompTypeCb(Widget w, XtPointer client_data, XtPointer call_data)
{
	nCompressionType = (int)client_data;

	XtVaSetValues(comptypebutton, XtNlabel, comp_type_menu[nCompressionType].Label, NULL);
}

void MesgSrcCb(Widget w, XtPointer client_data, XtPointer call_data)
{
	source = (int)client_data;

	XtVaSetValues(mesgsrcbutton, XtNlabel, mesg_src_menu[source].Label, NULL);
}

void FileTypeCb(Widget w, XtPointer client_data, XtPointer call_data)
{
	fileType = (int)client_data;

	XtVaSetValues(filetypebutton, XtNlabel, file_type_menu[fileType].Label, NULL);
}

int main(int argc, char **argv)
{
	static XtCallbackRec callback[2];
	char *s;

	if ((s = getenv("MYCALL")) == NULL)
		strcpy(myCall, "NOCALL");
	else
		strcpy(myCall, s);

	if ((s = getenv("BANNER")) == NULL)
	{
		if ((s = getenv("HOME")) != NULL)
		{
			strcpy(banner, s);
			strcat(banner, "/.sig");
		}
		else
		{
			strcpy(banner, ".sig");
		}
	}
	else
		strcpy(banner, s);

	if (argc > 1)
		strcpy(toCall, argv[1]);
	else
		strcpy(toCall, "");

	if (argc > 2)
		strcpy(title, argv[2]);
	else
		strcpy(title, "");
	
	toplevel = XtAppInitialize(&app_context, "Xpb", NULL, 0, &argc, argv,
				NULL, shell_args, XtNumber(shell_args));
	XtVaSetValues(toplevel, XtNtitle, VERSION_STRING, NULL);

	dpy  = XtDisplay(toplevel);

	XtGetApplicationResources(toplevel, &resources,
				resource_list, XtNumber(resource_list),
				NULL, ZERO);

	compwindow = XtCreateManagedWidget("composite", formWidgetClass,
				toplevel, form_args, XtNumber(form_args));

	callback[0].callback = QuitCb;
	callback[0].closure  = toplevel;
	button_args[0].value = (XtArgVal)callback;
	button_args[1].value = (XtArgVal)"Quit";
	button_args[2].value = (XtArgVal)resources.button_font;
	quitbutton = XtCreateManagedWidget("quit", commandWidgetClass,
				compwindow, button_args, XtNumber(button_args));

	callback[0].callback = PrepareCb;
	callback[0].closure  = toplevel;
	button_args[0].value = (XtArgVal)callback;
	button_args[1].value = (XtArgVal)"Prepare";
	button_args[3].value = (XtArgVal)quitbutton;
	preparebutton = XtCreateManagedWidget("prepare", commandWidgetClass,
				compwindow, button_args, XtNumber(button_args));

	label_args[0].value = (XtArgVal)quitbutton;
	label_args[1].value = (XtArgVal)"From:";
	label_args[2].value = (XtArgVal)75;
	label_args[3].value = (XtArgVal)resources.label_font;
	fromlabel = XtCreateManagedWidget("froml", labelWidgetClass,
				compwindow, label_args, XtNumber(label_args));

	label_args[0].value = (XtArgVal)fromlabel;
	label_args[1].value = (XtArgVal)"To:";
	tolabel = XtCreateManagedWidget("tol ", labelWidgetClass,
				compwindow, label_args, XtNumber(label_args));

	label_args[0].value = (XtArgVal)tolabel;
	label_args[1].value = (XtArgVal)"Title:";
	titlelabel = XtCreateManagedWidget("titlel ", labelWidgetClass,
				compwindow, label_args, XtNumber(label_args));

	label_args[0].value = (XtArgVal)titlelabel;
	label_args[1].value = (XtArgVal)"Key Words:";
	keywordslabel = XtCreateManagedWidget("keywordsl", labelWidgetClass,
				compwindow, label_args, XtNumber(label_args));

	label_args[0].value = (XtArgVal)keywordslabel;
	label_args[1].value = (XtArgVal)"File Type";
	filetypelabel = XtCreateManagedWidget("filetypel", labelWidgetClass,
				compwindow, label_args, XtNumber(label_args));

	label_args[0].value = (XtArgVal)filetypelabel;
	label_args[1].value = (XtArgVal)"File Type Description:";
	label_args[2].value = (XtArgVal)140;
	filedesclabel = XtCreateManagedWidget("filedescl", labelWidgetClass,
				compwindow, label_args, XtNumber(label_args));

	label_args[0].value = (XtArgVal)filedesclabel;
	label_args[1].value = (XtArgVal)"Compression Type";
	comptypelabel = XtCreateManagedWidget("comptypel", labelWidgetClass,
				compwindow, label_args, XtNumber(label_args));

	label_args[0].value = (XtArgVal)comptypelabel;
	label_args[1].value = (XtArgVal)"Message Source";
	mesgsrclabel = XtCreateManagedWidget("mesgsrcl", labelWidgetClass,
				compwindow, label_args, XtNumber(label_args));

	callback[0].callback = BannerCb;
	callback[0].closure  = toplevel;
	button_args[0].value = (XtArgVal)callback;
	button_args[1].value = (XtArgVal)"Append Banner";
	button_args[3].value = (XtArgVal)NULL;
	button_args[4].value = (XtArgVal)mesgsrclabel;
	bannerbutton = XtCreateManagedWidget("banner", commandWidgetClass,
				compwindow, button_args, XtNumber(button_args));

	window_args[0].value = (XtArgVal)bannerbutton;
	window_args[1].value = (XtArgVal)WhitePixel(dpy, DefaultScreen(dpy));
	window_args[2].value = (XtArgVal)resources.text_font;
	datawindow = XtCreateManagedWidget("data", asciiTextWidgetClass,
				compwindow, window_args, XtNumber(window_args));

	text_args[0].value = (XtArgVal)fromlabel;
	text_args[1].value = (XtArgVal)quitbutton;
	text_args[2].value = (XtArgVal)32;
	text_args[3].value = (XtArgVal)320;
	text_args[4].value = (XtArgVal)myCall;
	text_args[5].value = (XtArgVal)resources.button_font;
	fromtext = XtCreateManagedWidget("fromt", asciiTextWidgetClass,
				compwindow, text_args, XtNumber(text_args));

	text_args[0].value = (XtArgVal)tolabel;
	text_args[1].value = (XtArgVal)fromlabel;
	text_args[4].value = (XtArgVal)toCall;
	totext = XtCreateManagedWidget("tot", asciiTextWidgetClass,
				compwindow, text_args, XtNumber(text_args));

	text_args[0].value = (XtArgVal)titlelabel;
	text_args[1].value = (XtArgVal)tolabel;
	text_args[2].value = (XtArgVal)64;
	text_args[3].value = (XtArgVal)500;
	text_args[4].value = (XtArgVal)title;
	titletext = XtCreateManagedWidget("titlet", asciiTextWidgetClass,
				compwindow, text_args, XtNumber(text_args));

	text_args[0].value = (XtArgVal)keywordslabel;
	text_args[1].value = (XtArgVal)titlelabel;
	text_args[4].value = (XtArgVal)keywords;
	keywordstext = XtCreateManagedWidget("keywordst", asciiTextWidgetClass,
				compwindow, text_args, XtNumber(text_args));

	text_args[0].value = (XtArgVal)filedesclabel;
	text_args[1].value = (XtArgVal)filetypelabel;
	text_args[2].value = (XtArgVal)64;
	text_args[3].value = (XtArgVal)435;
	text_args[4].value = (XtArgVal)filetype_desc;
	filedesctext = XtCreateManagedWidget("filetypet", asciiTextWidgetClass,
				compwindow, text_args, XtNumber(text_args));

	menu_button_args[0].value = (XtArgVal)file_type_menu[0].Label;
	menu_button_args[1].value = (XtArgVal)filetypelabel;
	menu_button_args[2].value = (XtArgVal)keywordslabel;
	menu_button_args[3].value = (XtArgVal)175;
	menu_button_args[4].value = (XtArgVal)resources.button_font;
	filetypebutton = XtCreateManagedWidget("filetypeb", menuButtonWidgetClass,
				compwindow, menu_button_args, XtNumber(menu_button_args));

	createMenuPopup(filetypebutton, resources.menu_font, file_type_menu,
				XtNumber(file_type_menu), FileTypeCb);

	menu_button_args[0].value = (XtArgVal)comp_type_menu[0].Label;
	menu_button_args[1].value = (XtArgVal)comptypelabel;
	menu_button_args[2].value = (XtArgVal)filedesclabel;
	menu_button_args[3].value = (XtArgVal)110;
	comptypebutton = XtCreateManagedWidget("comptypeb", menuButtonWidgetClass,
				compwindow, menu_button_args, XtNumber(menu_button_args));

	createMenuPopup(comptypebutton, resources.menu_font, comp_type_menu,
				XtNumber(comp_type_menu), CompTypeCb);

	menu_button_args[0].value = (XtArgVal)mesg_src_menu[0].Label;
	menu_button_args[1].value = (XtArgVal)mesgsrclabel;
	menu_button_args[2].value = (XtArgVal)comptypelabel;
	mesgsrcbutton = XtCreateManagedWidget("mesgsrcb", menuButtonWidgetClass,
				compwindow, menu_button_args, XtNumber(menu_button_args));

	createMenuPopup(mesgsrcbutton, resources.menu_font, mesg_src_menu,
				XtNumber(mesg_src_menu), MesgSrcCb);

	createMessagePopup(resources.bold_font, resources.button_font);

	XtRealizeWidget(toplevel);
				
	XtAppMainLoop(app_context);
	
	return(0);
}

